use std::{
    io,
    process::{Command, Output},
};

#[non_exhaustive]
/// Defines a set of commands to be run in order to finish package installation
pub enum Hooks {
    /// Runs `makewhatis` to update the mandoc database if the package contains
    /// any Unix man pages
    Man,
    /// Runs `glib-compile-schemas` if the package contains any GLib schema files
    GlibSchema,
    /// runs `install-info` to update the GNU Texinfo database
    Info(String),
}

impl Hooks {
    pub fn run(&self) -> Result<Output, io::Error> {
        match self {
            Self::Man => makeinfo(),
            Self::GlibSchema => compile_schemas(),
            Self::Info(path) => install_info(path),
        }
    }
}

fn makeinfo() -> Result<Output, io::Error> {
    Command::new("makewhatis").output()
}

fn compile_schemas() -> Result<Output, io::Error> {
    Command::new("glib-compile-schemas")
        .arg("/usr/share/glib-2.0/schemas")
        .output()
}

fn install_info(path: &str) -> Result<Output, io::Error> {
    Command::new("install-info").arg(path).output()
}
